/*
 * Copyright (C) 2004 emuWorks
 * http://games.technoplaza.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package chtdecoder.view;

import java.awt.Button;
import java.awt.FileDialog;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Label;
import java.awt.List;
import java.awt.Panel;
import java.awt.ScrollPane;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import chtdecoder.CHTDecoder;

import chtdecoder.datastructures.CHTCodeIterator;

import chtdecoder.exceptions.CHTFileException;

import chtdecoder.tools.AbstractCHTFileReader;

/**
 * An applet GUI for the CHTDecoder program.
 *
 * @author John David Ratliff
 * @version 1.2, 09/21/04
 */
public class CHTApplet extends java.applet.Applet {
    private Frame frame;
    private FileDialog dialog;
    private Panel buttonPanel;
    private List codeList;
    private ScrollPane codeScroller;
    private Label copyrightLabel;
    private Label nameLabel;
    private Button openButton;
    private Button saveButton;
    private Panel textPanel;

    /**
     * Creates a new CHTApplet object.
     */
    public CHTApplet() {
        this.frame = new Frame();
    }

    /**
     * Creates a new CHTApplet object.
     *
     * @param frame The parent frame.
     */
    public CHTApplet(Frame frame) {
        this.frame = frame;
    }

    /**
     * Initializes the applet.
     */
    public void init() {
        initComponents();
    }

    /**
     * Initializes the applet's components.
     */
    private void initComponents() {
        GridBagConstraints gridBagConstraints;

        buttonPanel = new Panel();
        openButton = new Button();
        saveButton = new Button();
        textPanel = new Panel();
        nameLabel = new Label();
        copyrightLabel = new Label();
        codeScroller = new ScrollPane();
        codeList = new List();

        setLayout(new GridBagLayout());

        openButton.setLabel("Open");
        openButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    openButtonAction();
                }
            });
        buttonPanel.add(openButton);

        saveButton.setLabel("Save");
        saveButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    saveButtonAction();
                }
            });
        buttonPanel.add(saveButton);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(5, 5, 5, 5);
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 0.05;
        add(buttonPanel, gridBagConstraints);

        textPanel.setLayout(new GridBagLayout());

        nameLabel.setText(CHTDecoder.APP_NAME + " " +
                          CHTDecoder.APP_VERSION);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        textPanel.add(nameLabel, gridBagConstraints);

        copyrightLabel.setText(CHTDecoder.APP_COPYRIGHT);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        textPanel.add(copyrightLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        add(textPanel, gridBagConstraints);

        codeList.setFont(new Font("Monospaced", Font.PLAIN, 14));
        codeScroller.add(codeList);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(5, 5, 5, 5);
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 0.95;
        add(codeScroller, gridBagConstraints);
    }

    /**
     * Builds the FileDialog used to open and save files.
     */
    private void buildFileDialog() {
        this.dialog = new FileDialog(this.frame, ".CHT Files");
    }

    /**
     * Callback when the open button is pressed.
     */
    private void openButtonAction() {
        if (this.dialog == null) {
            buildFileDialog();
        }

        dialog.setMode(FileDialog.LOAD);
        dialog.show();
        
        if (dialog.getFile() == null) {
            return;
        }

        File file = new File(dialog.getDirectory() + File.separator +
                             dialog.getFile());

        try {
            CHTCodeIterator iterator = AbstractCHTFileReader.readCHTFile(file);
            this.codeList.removeAll();

            while (iterator.hasMore()) {
                this.codeList.add(iterator.next().toString());
            }
        } catch (CHTFileException e) {
            MessageDialog message = new MessageDialog(this.frame,
                                                      ".CHT File Error",
                                                      e.getMessage());
            message.show();
        }
    }

    /**
     * Callback when the save button is pressed.
     */
    private void saveButtonAction() {
        if (this.dialog == null) {
            buildFileDialog();
        }

        dialog.setMode(FileDialog.SAVE);
        dialog.show();
        
        if (dialog.getFile() == null) {
            return;
        }

        File file = new File(dialog.getDirectory() + File.separator +
                             dialog.getFile());
        
        try {
            PrintWriter writer = new PrintWriter(new FileWriter(file));
            String[] codes = this.codeList.getItems();

            for (int pos = 0; pos < codes.length; pos++) {
                writer.println(codes[pos]);
            }

            writer.flush();
            writer.close();
        } catch (IOException e) {
            MessageDialog message = new MessageDialog(this.frame,
                                                      "IO Error",
                                                      e.getMessage());
            message.show();
        }
    }

    /**
     * Returns a String representation of this applet.
     *
     * @return A String representation.
     */
    public String toString() {
        return "CHTApplet";
    }
}
